"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const isValidPath = require("is-valid-path");
const isGlob = require("is-glob");
const path_1 = require("path");
const common_1 = require("@graphql-toolkit/common");
const filter_document_kind_1 = require("./filter-document-kind");
const document_from_string_1 = require("./document-from-string");
function filterFiles(files) {
    return files.filter(file => !file.endsWith('.d.ts') && !file.endsWith('.spec.ts') && !file.endsWith('.spec.js') && !file.endsWith('.test.ts') && !file.endsWith('.test.js'));
}
function isUrl(pointer) {
    return typeof pointer === 'string' && /^https?\:\/\//i.test(pointer);
}
async function loadTypedefsUsingLoaders(loaders, pointerOrPointers, options = {}, filterKinds = [], cwd = process.cwd()) {
    const pointers = common_1.asArray(pointerOrPointers);
    const loadPromises$ = [];
    const found = [];
    const foundGlobs = [];
    for (const pointer of pointers) {
        if (common_1.isDocumentString(pointer)) {
            loadPromises$.push(Promise.resolve().then(async () => {
                const docs = document_from_string_1.documentFromString(pointer, filterKinds);
                found.push(...docs);
            }));
        }
        else if (isUrl(pointer)) {
            loadPromises$.push(Promise.resolve().then(async () => {
                let content = await loadSingleFile(loaders, pointer, options, cwd);
                content = filter_document_kind_1.filterKind(content, filterKinds);
                if (content && content.definitions && content.definitions.length > 0) {
                    found.push({
                        location: pointer,
                        document: content,
                    });
                }
            }));
        }
        else if (isValidPath(pointer)) {
            const fixedPath = common_1.fixWindowsPath(pointer);
            const relevantFiles = filterFiles(common_1.asArray(fixedPath));
            for (const filePath of relevantFiles) {
                loadPromises$.push(Promise.resolve().then(async () => {
                    let content = await loadSingleFile(loaders, filePath, options, cwd);
                    content = filter_document_kind_1.filterKind(content, filterKinds);
                    if (content && content.definitions && content.definitions.length > 0) {
                        found.push({
                            location: filePath,
                            document: content,
                        });
                    }
                }));
            }
        }
        else if (isGlob(pointer)) {
            foundGlobs.push(pointer);
        }
    }
    if (foundGlobs.length > 0) {
        if (options.ignore) {
            const ignoreList = common_1.asArray(options.ignore)
                .map(g => `!(${g})`)
                .map(p => common_1.fixWindowsPath(p));
            if (ignoreList.length > 0) {
                foundGlobs.push(...ignoreList);
            }
        }
        const globby = eval(`require('globby')`);
        loadPromises$.push(Promise.resolve().then(async () => {
            const paths = await globby(foundGlobs, { cwd, absolute: true });
            await Promise.all(paths.map(async (path) => {
                const filePath = common_1.fixWindowsPath(path);
                let content = await loadSingleFile(loaders, filePath, options, cwd);
                content = filter_document_kind_1.filterKind(content, filterKinds);
                if (content && content.definitions && content.definitions.length > 0) {
                    found.push({
                        location: filePath,
                        document: content,
                    });
                }
            }));
        }));
    }
    await Promise.all(loadPromises$);
    if (found.length === 0) {
        throw new Error(`Unable to find any GraphQL type definitions for the following pointers: ${pointers.join(', ')}`);
    }
    return found.sort((left, right) => left.location.localeCompare(right.location));
}
exports.loadTypedefsUsingLoaders = loadTypedefsUsingLoaders;
async function loadSingleFile(loaders, pointer, options = {}, cwd = process.cwd()) {
    const fullPath = isUrl(pointer) ? pointer : common_1.fixWindowsPath(path_1.isAbsolute(pointer) ? pointer : path_1.resolve(cwd, pointer));
    try {
        for (const loader of loaders) {
            const canLoad = await loader.canLoad(fullPath, options);
            if (canLoad) {
                const found = await loader.load(fullPath, options);
                if (found) {
                    return found.document;
                }
            }
        }
    }
    catch (e) {
        common_1.debugLog(`Failed to find any GraphQL type definitions in: ${pointer} - ${e.message}`);
        throw e;
    }
    return null;
}
exports.loadSingleFile = loadSingleFile;
//# sourceMappingURL=load-typedefs.js.map