import * as isValidPath from 'is-valid-path';
import * as isGlob from 'is-glob';
import { isAbsolute, resolve as resolvePath } from 'path';
import { asArray, isDocumentString, debugLog, fixWindowsPath } from '@graphql-toolkit/common';
import { filterKind } from './filter-document-kind';
import { documentFromString } from './document-from-string';
function filterFiles(files) {
    return files.filter(file => !file.endsWith('.d.ts') && !file.endsWith('.spec.ts') && !file.endsWith('.spec.js') && !file.endsWith('.test.ts') && !file.endsWith('.test.js'));
}
function isUrl(pointer) {
    return typeof pointer === 'string' && /^https?\:\/\//i.test(pointer);
}
export async function loadTypedefsUsingLoaders(loaders, pointerOrPointers, options = {}, filterKinds = [], cwd = process.cwd()) {
    const pointers = asArray(pointerOrPointers);
    const loadPromises$ = [];
    const found = [];
    const foundGlobs = [];
    for (const pointer of pointers) {
        if (isDocumentString(pointer)) {
            loadPromises$.push(Promise.resolve().then(async () => {
                const docs = documentFromString(pointer, filterKinds);
                found.push(...docs);
            }));
        }
        else if (isUrl(pointer)) {
            loadPromises$.push(Promise.resolve().then(async () => {
                let content = await loadSingleFile(loaders, pointer, options, cwd);
                content = filterKind(content, filterKinds);
                if (content && content.definitions && content.definitions.length > 0) {
                    found.push({
                        location: pointer,
                        document: content,
                    });
                }
            }));
        }
        else if (isValidPath(pointer)) {
            const fixedPath = fixWindowsPath(pointer);
            const relevantFiles = filterFiles(asArray(fixedPath));
            for (const filePath of relevantFiles) {
                loadPromises$.push(Promise.resolve().then(async () => {
                    let content = await loadSingleFile(loaders, filePath, options, cwd);
                    content = filterKind(content, filterKinds);
                    if (content && content.definitions && content.definitions.length > 0) {
                        found.push({
                            location: filePath,
                            document: content,
                        });
                    }
                }));
            }
        }
        else if (isGlob(pointer)) {
            foundGlobs.push(pointer);
        }
    }
    if (foundGlobs.length > 0) {
        if (options.ignore) {
            const ignoreList = asArray(options.ignore)
                .map(g => `!(${g})`)
                .map(p => fixWindowsPath(p));
            if (ignoreList.length > 0) {
                foundGlobs.push(...ignoreList);
            }
        }
        const globby = eval(`require('globby')`);
        loadPromises$.push(Promise.resolve().then(async () => {
            const paths = await globby(foundGlobs, { cwd, absolute: true });
            await Promise.all(paths.map(async (path) => {
                const filePath = fixWindowsPath(path);
                let content = await loadSingleFile(loaders, filePath, options, cwd);
                content = filterKind(content, filterKinds);
                if (content && content.definitions && content.definitions.length > 0) {
                    found.push({
                        location: filePath,
                        document: content,
                    });
                }
            }));
        }));
    }
    await Promise.all(loadPromises$);
    if (found.length === 0) {
        throw new Error(`Unable to find any GraphQL type definitions for the following pointers: ${pointers.join(', ')}`);
    }
    return found.sort((left, right) => left.location.localeCompare(right.location));
}
export async function loadSingleFile(loaders, pointer, options = {}, cwd = process.cwd()) {
    const fullPath = isUrl(pointer) ? pointer : fixWindowsPath(isAbsolute(pointer) ? pointer : resolvePath(cwd, pointer));
    try {
        for (const loader of loaders) {
            const canLoad = await loader.canLoad(fullPath, options);
            if (canLoad) {
                const found = await loader.load(fullPath, options);
                if (found) {
                    return found.document;
                }
            }
        }
    }
    catch (e) {
        debugLog(`Failed to find any GraphQL type definitions in: ${pointer} - ${e.message}`);
        throw e;
    }
    return null;
}
//# sourceMappingURL=load-typedefs.js.map